/* 	Version 2.2 eventstudy April 28, 2013
	Created by Jessica Laird e-mail at jalpernl@gmail.com
	Edited slightly by Michael Stepner
	This program creates a connected scatterplot of an event study
	The command "eventstudy" takes one outcome variable, and then in options, an event variable, a time variable and a unit variable.
	
	eventvar(varname): the variable that specifies the event, for example leave_event [MUST be specified]
	timevar(varname): the variable that specifies the time variable, for example year [MUST be specified]
	unitvar(varname): the variable that specifies the unitvariable [MUST be specified]
	pickevent(string): Takes stack, first, or last. This specifies if there are multiple events in the time period for a given unit, which should be choosen.  [MUST be specified] 
				"stack" says that should use all events.
				"first" picks the first event.
				"last" picks the last event. 
	preyrs(integer): Gives the number of pre-years (or whatever time unit specified by timevar) - defaults to 3.  
	postyrs(integer): Gives the number of post-years (or whatever time unit specified by timevar) after the event - defaults to 3
	balanced: Says whether wants a balanced sample - if so, resticts to same number of observations for year. Default is unbalanced
	controls(varlist): Gives a list of control variables the outcome variable to be regressed on to find the coefficients
	weight(varname): Specifies a weight variable for regressions to run on
	log(string): Creates a log saved as whatever is specified in string
	absorb(varname): Runs regressions using areg and absrobing the specified variable
	noaddmean : Makes the coef of the event year equal to zero, and has all coefficients in relation to that
	noreportcoef : Doesn't report the event year coefficients and their standard errors
	saving(string) : Saves scatterplot as specified string.
	savedata(string) : Saves dataset underlying the scatterplot
	eventyr_plus1: Specifies that the first "after event" year is at 1 instead of 0 (default).   
	testval(string): Tests whether the average of the after coefficients minus the average of the before coefficients is equal to number specified 
		For coding purposes takes a string, but will only take a number argument
		Doesn't work for odd numbers of total years, or total years more than 12 - though easy to change.  Search for test and add in test that want.
	testvalone(string): Tests whether the after minus the before dot is equal to the number specified.  
		For coding purposes takes a string, but will only take a number argument
	cluster(varname): Clusters standard errors according to variable specified. 
	* Specifies that any options for graphs (such as the y scale, y title) can be specified.


	Command is of eclass type and stores:
		- the number of events in e(N_events)
		- the after minus before dot coefficient in e(coef_aft_dot)
		- the after minus before dot standard error in e(se_aft_dot)
		- if applicable, the p-val for the test specified by testvalone() in e(testvalone_p)

	example code:
	eventstudy mean_score if year>1993, eventvar(leave_event) timevar(year) unitvar(grade_id) pickevent(stack) preyrs(3) postyrs(2) controls(_I*) balanced noaddmean testval(.05)
	display e(coef_aft_dot)
	display e(se_aft_dot)

*/

capture program drop eventstudy
program define eventstudy, eclass

	version 11

	set more off
	
	syntax varname [if/], eventvar(varname) timevar(varname) unitvar(varname) preyrs(integer) postyrs(integer) ///
		[ controls(varlist) weight(varname) absorb(varname) cluster(varname) ///
		log(string) saving(string) savedata(string) ///
		pickevent(string) testval(string) testvalone(string) ///
		balanced eventyr_plus1  NOADdmean NOREportcoef nograph * ]

	local outcomevar `varlist'

	if "`cluster'"!=""{
		local ccluster `"cluster(`cluster')"'
	}
	
	if "`pickevent'"!="stack" & "`pickevent'"!="first" &"`pickevent'"!="last"{
		di as error `"Not valid argument for pickevent - choose stack, first, or last"'
		exit
	}

	if "`if'"!="" {
		local andif `" & (`if')"'
	}

	if "`testval'"!="" {
		local testval= real(string(`testval'))
		if "`testval'"=="" {
			di as error `"Not valid argument for testval, choose a real number"'
			exit
		}
	} 

	if "`testvalone'"!="" {
		local testvalone= real(string(`testvalone'))
		if "`testvalone'"=="" {
			di as error `"Not valid argument for testvalone, choose a real number"'
			exit
		}
	}
	
	if "`pickevent'"=="stack" | "`savedata'"!="" {
		preserve
	}

	tempvar temp leave_yr bal_outcome event_coeffs event_se event_graph leave_yr num_eve new_unit_id

	if "`log'"!="" {
		cap log close
		log using "`log'", replace
	}

	if "`pickevent'"=="stack" {
		
		if "`if'"!="" {
			qui keep if `if'
		}
		* Creates an for each event in unitvar
		qui g `temp' = `timevar' if `eventvar'==1
		qui count if `temp'!=.
		local i=1
		while r(N)!=0 {
			qui bys `unitvar': egen leave_yr`i' = min(`temp')
			qui replace `temp'=. if `temp'==leave_yr`i'
			qui count if `temp'!=.
			local i = `i'+1
		}
	
		qui reshape long leave_yr, i(`unitvar' `timevar') j(`num_eve')
		qui gen `leave_yr'=leave_yr
				
		qui drop if `leave_yr'==.
		qui egen `new_unit_id'= group(`unitvar' `leave_yr') 
		keep `outcomevar' `controls' `weight' `absorb' `timevar' `unitvar' `leave_yr' `new_unit_id' `cluster' 
	} 

	else if "`pickevent'"=="first" {
		qui g `temp' = `timevar' if `eventvar'==1 `andif'
		if "`if'"!="" {
			qui bys `unitvar': egen `leave_yr' = min(`temp') if `if'
		}
		else {
			qui bys `unitvar': egen `leave_yr' = min(`temp') 
		}
		
		drop `temp'
	} 
	else if "`pickevent'"=="last" {
		qui g `temp' = `timevar' if `eventvar'==1 `andif'
		if "`if'"!="" {
			qui bys `unitvar': egen `leave_yr' = max(`temp') if `if'
		}
		else {
			qui bys `unitvar': egen `leave_yr' = max(`temp') 
		}
		
		drop `temp'
	}

	if "`weight'"!=""{
		local weight `"[w=`weight']"'
	}

	qui g event_yr = `timevar'-`leave_yr' 
	qui replace event_yr = . if event_yr<-`preyrs' | event_yr>`postyrs' 
		
	* Creates a dummy for each event year.  Omits the year before the event. In the case of eventyr_plus1, that is the "0" year, and otherwise it is the "-1" year.
	if "`eventyr_plus1'"!=""{
		char event_yr[omit] 0
	}
	else {
		char event_yr[omit] -1
	}
	qui xi, prefix(E) i.event_yr, if event_yr>=-`preyrs' & event_yr<=`postyrs' 

	local total_event = `preyrs'+`postyrs'+1

	* Stores the number the after dot coefficient will be in the local `aft_coef_num'.  In the case of eventyr_plus1 this is preyrs+2, and otherwise it is preyrs+1. 
	if "`eventyr_plus1'"!=""{
		local aft_coef_num = `preyrs'+2
	}
	else {
		local aft_coef_num = `preyrs'+1
	}

	* If unbalanced just runs the regression
	if ("`balanced'"==""){
		if ("`absorb'"==""){
			if "`pickevent'"!= "stack" {			
				qui reg `outcomevar' Ee* `controls' `weight' if event_yr>=-`preyrs' & event_yr<=`postyrs'  `andif', `ccluster'
			}
			* If stacked, then sample is already resticted by using a keep statement so no need to use andif restriction in regression
			else {
				qui reg `outcomevar' Ee* `controls' `weight' if event_yr>=-`preyrs' & event_yr<=`postyrs', `ccluster'
			}
		}
		else {
			if "`pickevent'"!= "stack"{
				qui areg `outcomevar' Ee* `controls' `weight' if event_yr>=-`preyrs' & event_yr<=`postyrs'  `andif', a(`absorb') `ccluster'
			}
			else {
				qui areg `outcomevar' Ee* `controls' `weight' if event_yr>=-`preyrs' & event_yr<=`postyrs', a(`absorb') `ccluster'
			}
		}
	}
	
	* If balanced, creates a variable that is equal 0 if one of the event years has missing event variable information.  
	* Then runs regression on only the observations that don't have any missing event years.
	else {
		
		if "`pickevent'"!= "stack"{
			qui xtset `unitvar'  `timevar'
			sort `unitvar'  `timevar'
			qui g `temp' = 1 if event_yr==0 `andif'
		} 
		else {
			qui xtset `new_unit_id'  `timevar'
			sort `new_unit_id' `timevar'
			qui g `temp' = 1 if event_yr==0 
		}
		
		if `preyrs'>=2{
			forvalues i=2/`preyrs' {
				qui replace `temp'=0 if (l`i'.`outcomevar'==. | l`i'.event_yr==.) & event_yr==0
			}
		} 

		qui replace `temp'=0 if (l.`outcomevar'==. | l.event_yr==.) & event_yr==0
		
		if `postyrs'>=2{
			forvalues i=2/`postyrs' {
				qui replace `temp'=0 if (f`i'.`outcomevar'==.  | f`i'.event_yr==.) & event_yr==0
			}
		} 
		
		if `postyrs'>=1 {
			qui replace `temp'=0 if (f.`outcomevar'==. | f.event_yr==.) & event_yr==0 
		}
		
		qui replace `temp'=0 if `outcomevar'==.& event_yr==0 
		
		if "`pickevent'"!= "stack"{
			qui bys `unitvar': egen `bal_outcome' = mean(`temp')
		} 
		else {
			qui bys `new_unit_id': egen `bal_outcome' = mean(`temp')
		}

		if ("`absorb'"==""){
			if "`pickevent'"!= "stack"{
				qui reg `outcomevar' Ee* `controls' `weight' if event_yr>=-`preyrs' & event_yr<=`postyrs'  `andif' & `bal_outcome'==1, `ccluster'
			}
			else {
				qui reg `outcomevar' Ee* `controls' `weight' if event_yr>=-`preyrs' & event_yr<=`postyrs' & `bal_outcome'==1, `ccluster'
			}
		}
		else {
			if "`pickevent'"!= "stack"{
				qui areg `outcomevar' Ee* `controls' `weight' if event_yr>=-`preyrs' & event_yr<=`postyrs'  `andif' & `bal_outcome'==1, a(`absorb') `ccluster'
			}
			else {
				qui areg `outcomevar' Ee* `controls' `weight' if event_yr>=-`preyrs' & event_yr<=`postyrs' & `bal_outcome'==1, a(`absorb') `ccluster'
			}
		}
	}

	* Runs f-tests on coefficients for testval option.   
	if "`testval'"!=""{
		if mod(`total_event',2)==0&`total_event'<=12{
			display _newline as text "Test if average of after dots minus before dots = " "`testval'"
			if `total_event'==4{
				test (_b[Eevent_yr_3] + _b[Eevent_yr_4])/2 - (_b[Eevent_yr_1])/2=`testval'
			}
			else if `total_event'==6 {
				test (_b[Eevent_yr_4] + _b[Eevent_yr_5] + _b[Eevent_yr_6])/3 - (_b[Eevent_yr_1] + _b[Eevent_yr_2])/3=`testval'
			}
			else if `total_event'==8{
				test (_b[Eevent_yr_5] + _b[Eevent_yr_6]+_b[Eevent_yr_7]+_b[Eevent_yr_8])/4 - (_b[Eevent_yr_1] + _b[Eevent_yr_2]+_b[Eevent_yr_3])/4=`testval'
			}
			else if `total_event'==10{
				test (_b[Eevent_yr_6] + _b[Eevent_yr_7]+_b[Eevent_yr_8]+_b[Eevent_yr_9]+_b[Eevent_yr_10])/5 - (_b[Eevent_yr_1] + _b[Eevent_yr_2]+_b[Eevent_yr_3]+_b[Eevent_yr_4])/5=`testval'
			}
			else if `total_event'==12{
				test (_b[Eevent_yr_7]+_b[Eevent_yr_8]+_b[Eevent_yr_9]+_b[Eevent_yr_10]+_b[Eevent_yr_11]+_b[Eevent_yr_12])/6 - (_b[Eevent_yr_1] + _b[Eevent_yr_2]+_b[Eevent_yr_3]+_b[Eevent_yr_4]+_b[Eevent_yr_5])/6=`testval'
			}
		}
		else if mod(`total_event',2)!=0 {
			display _newline as text "Can't test for odd number of years"			
		}
		else if `total_event'>12 {
			display _newline as text "Can't test for total number of years more than 12 - Easy to change ado file to accomodate though."				
		}
	}
	
	* Tests whether after dot-before dot is equal to testvalone.
	if "`testvalone'"!=""{
		display _newline as text "Test if after dot - before dot = " "`testvalone'"
		test _b[Eevent_yr_`aft_coef_num'] =`testvalone'
		
		* Store pval of test in local to be returned in e()
		local testvalone_p=r(p)
	}
	
	qui g `event_coeffs' = .
	qui g `event_se' = .
	 
	local i=1
	local sum_coef=0

	* Stores coefficients in obs 1 to total number of event years.
	* If the event year is equal to -1 (number of times goes through loop equals number of preyrs) then sets coef and se equal to zero.
	* If have eventyr_plus1, then then sets coef and se equal to zero when the event year is equal to 0 (number of times goes through loop equals number of preyrs+1)
	qui count if e(sample)
	local total=r(N)
	foreach var of varlist Ee*{
		if(`i'==`preyrs' & "`eventyr_plus1'"=="" ){
			qui replace `event_coeffs'=0 in `i'
			qui replace `event_se'=0 in `i'	
			local i=`i'+1
			qui count if `var'==1& e(sample)
			di _newline "Number of Events: " r(N)
			local N_events=r(N)
		}

		if(`i'==`preyrs'+1 & "`eventyr_plus1'"!=""){
			qui replace `event_coeffs'=0 in `i'
			qui replace `event_se'=0 in `i'	
			local i=`i'+1
			qui count if `var'==1& e(sample)
			di _newline "Number of Events: " r(N) 
			local N_events=r(N)
		}

		if _se[`var']==0   {
			di as error `"Collinearty problem.  Drop a control"'
			exit
		} 
			
		qui replace `event_coeffs' = _b[`var'] in `i'	
		qui replace `event_se' = _se[`var'] in `i'
		qui count if `var'==1& e(sample)
		local sum_coef=`sum_coef'+_b[`var']*r(N)/`total'
		
		local i=`i'+1
	}
	

	* If noaddmean is specified then makes the average of the coefficients equal to the average of the outcome variable (on same sample that regressions ran on)

	if "`noaddmean'"==""{
		qui sum `outcomevar' `weight' if e(sample)
		local addmean=r(mean)-`sum_coef' 
		qui replace `event_coeffs'=`event_coeffs'+`addmean'
	}
	
	*** Display the after-before dot coeff and standard error, and store in locals to be returned in e()
	* Display the coef and se
	di _n "After coefficient minus before coefficient: " _b[Eevent_yr_`aft_coef_num']
	di "After coefficient minus before coefficient standard error: " _se[Eevent_yr_`aft_coef_num']
	
	* Store them in locals
	local coef_aft_dot=_b[Eevent_yr_`aft_coef_num'] 
	local se_aft_dot=_se[Eevent_yr_`aft_coef_num'] 
	
	
	qui gen `event_graph' = _n-1-`preyrs' in 1/`total_event'
	
	* Displays coefficient and se table
	if "`noreportcoef'"==""{
		di _n "      Event Year:  Coef:        Se:"
		list `event_graph' `event_coeffs' `event_se' if `event_coeffs' !=., noheader
	}

	* Plot graph
	if ("`graph'"!="nograph") scatter `event_coeffs' `event_graph', c(l) xtitle("Year Relative to Event") ytitle("`outcomevar'") `options'

	* Save graph
	if (`"`saving'"'!="") & ("`graph'"!="nograph") {
  		graph save "`saving'", replace 
  	}
	
	* Save data
	if ("`savedata'"!="") {
		drop if _n>`total_event'
		keep `event_graph' `event_coeffs' `event_se'
		rename `event_graph' event_time
		rename `event_coeffs' `outcomevar'_coeffs
		rename `event_se' `outcomevar'_se
		order event_time `outcomevar'_coeffs `outcomevar'_se
		notes: scatter `outcomevar'_coeffs event_time, c(l) xtitle("Year Relative to Event") ytitle("`outcomevar'") `options'
		save `"`savedata'"', replace
	}

	if ("`log'"!="") log close
	
	if ("`pickevent'"=="stack" | "`savedata'"!="") restore
	else drop Ee* event_yr
	
	* Clear e() results
	ereturn clear
	
	* Return e(coef_aft_dot), e(se_aft_dot), e(testvalone_p)
	ereturn scalar N_events=`N_events'
	ereturn scalar coef_aft_dot =`coef_aft_dot'
	ereturn scalar se_aft_dot =`se_aft_dot'
	if ("`testvalone_p'"!="") ereturn scalar testvalone_p =`testvalone_p'
end

